@@ -3,8 +3,10 @@ import json
 import logging
 
 from django.core.files.base import ContentFile
+from django.db import connection
 from django.db import IntegrityError, transaction
 from django.db.models import Q
+from django.db.utils import DatabaseError
 
 from twisted.internet.threads import deferToThread
 
@@ -13,6 +15,13 @@ from zope.interface import implements
 from lava_scheduler_app.models import Device, TestJob
 from lava_scheduler_daemon.jobsource import IJobSource
 
+try:
+    from psycopg2 import InterfaceError, OperationalError
+except ImportError:
+    class InterfaceError(Exception):
+        pass
+    class OperationalError(Exception):
+        pass
 
 
 class DatabaseJobSource(object):
@@ -24,8 +33,28 @@ class DatabaseJobSource(object):
     def getBoardList_impl(self):
         return [d.hostname for d in Device.objects.all()]
 
+    def deferForDB(self, func, *args, **kw):
+        def wrapper(*args, **kw):
+            try:
+                return func(*args, **kw)
+            except (DatabaseError, OperationalError, InterfaceError), error:
+                message = str(error)
+                if message == 'connection already closed' or \
+                   message.startswith(
+                    'terminating connection due to administrator command') or \
+                   message.startswith(
+                    'could not connect to server: Connection refused'):
+                    self.logger.warning(
+                        'Forcing reconnection on next db access attempt')
+                    if connection.connection:
+                        if not connection.connection.closed:
+                            connection.connection.close()
+                        connection.connection = None
+                raise
+        return deferToThread(wrapper, *args, **kw)
+
     def getBoardList(self):
-        return deferToThread(self.getBoardList_impl)
+        return self.deferForDB(self.getBoardList_impl)
 
     @transaction.commit_manually()
     def getJobForBoard_impl(self, board_name):
@@ -65,10 +94,7 @@ class DatabaseJobSource(object):
                     json_data = json.loads(job.definition)
                     json_data['target'] = device.hostname
                     transaction.commit()
-                    log_file = job.log_file
-                    log_file.file.close()
-                    log_file.open('wb')
-                    return json_data, log_file
+                    return json_data
             else:
                 # We don't really need to rollback here, as no modifying
                 # operations have been made to the database.  But Django is
@@ -79,22 +105,49 @@ class DatabaseJobSource(object):
                 return None
 
     def getJobForBoard(self, board_name):
-        return deferToThread(self.getJobForBoard_impl, board_name)
+        return self.deferForDB(self.getJobForBoard_impl, board_name)
+
+    @transaction.commit_on_success()
+    def getLogFileForJobOnBoard_impl(self, board_name):
+        device = Device.objects.get(hostname=board_name)
+        device.status = Device.IDLE
+        job = device.current_job
+        log_file = job.log_file
+        log_file.file.close()
+        log_file.open('wb')
+        return log_file
+
+    def getLogFileForJobOnBoard(self, board_name):
+        return self.deferForDB(self.getLogFileForJobOnBoard_impl, board_name)
 
     @transaction.commit_on_success()
     def jobCompleted_impl(self, board_name):
         self.logger.debug('marking job as complete on %s', board_name)
         device = Device.objects.get(hostname=board_name)
-        device.status = Device.IDLE
+        if device.status == Device.RUNNING:
+            device.status = Device.IDLE
+        elif device.status == Device.OFFLINING:
+            device.status = Device.OFFLINE
+        else:
+            self.logger.error(
+                "Unexpected device state in jobCompleted: %s" % device.status)
+            device.status = Device.IDLE
         job = device.current_job
         device.current_job = None
-        job.status = TestJob.COMPLETE
+        if job.status == TestJob.RUNNING:
+            job.status = TestJob.COMPLETE
+        elif job.status == TestJob.CANCELING:
+            job.status = TestJob.CANCELED
+        else:
+            self.logger.error(
+                "Unexpected job state in jobCompleted: %s" % job.status)
+            job.status = TestJob.COMPLETE
         job.end_time = datetime.datetime.utcnow()
         device.save()
         job.save()
 
     def jobCompleted(self, board_name):
-        return deferToThread(self.jobCompleted_impl, board_name)
+        return self.deferForDB(self.jobCompleted_impl, board_name)
 
     @transaction.commit_on_success()
     def jobOobData_impl(self, board_name, key, value):
@@ -106,5 +159,13 @@ class DatabaseJobSource(object):
             device.current_job.save()
 
     def jobOobData(self, board_name, key, value):
-        return deferToThread(self.jobOobData_impl, board_name, key, value)
+        return self.deferForDB(self.jobOobData_impl, board_name, key, value)
 
+    def jobCheckForCancellation_impl(self, board_name):
+        device = Device.objects.get(hostname=board_name)
+        device.status = Device.IDLE
+        job = device.current_job
+        return job.status != TestJob.RUNNING
+
+    def jobCheckForCancellation(self, board_name):
+        return self.deferForDB(self.jobCheckForCancellation_impl, board_name)
